@@ -17,7 +17,7 @@
 '''
 
 
-
+import datetime
 from dataclasses import dataclass
 import json
 import re as re
@@ -86,7 +86,6 @@ class MySQLDatabase:
         '''True if the credentials dictionary can be successfully compiled.'''
 
 
-
         self.data = {}
         self.settings = {}
         self.con = None
@@ -118,9 +117,15 @@ class MySQLDatabase:
         # print(f"self._tables:{self._tables}")
         value['tables'] = { k:v.summary for (k,v) in self._tables.items()}
         # value['tables'] = [x.summary for x in self._tables]
-        
+
         return value
 
+    # def save(self):
+    #     table:_db_table_type
+    #     for _,table in self._tables.items():
+    #         table.cache.save()
+        # _cfu.writer.to_json(f"{self.cache_path}/db_summary.json",self.summary)
+
     @property
     def __credentials(self):
         '''
@@ -365,15 +370,12 @@ class MySQLDatabase:
         if len(data.keys()) == 0:
             return False
         sql,args = insert_from_dict(data,table_name,self.database)
-
+        print(f"sql: {sql}")
+        print(f"args: {args}")
         result = self.run(sql,args)
         if result is True:
             return self.last_id()
 
-        
-        
-        # print(f"result:{(sql,args)}")
-
     def select_query(self,table_name:str)->_db_mysql_select_query_type:
         '''Create a new select query instance'''
         return SelectQuery(database=self,table_name=table_name)
@@ -689,7 +691,7 @@ class MySQLDatabase:
             -------------------------
             `table_name` {str}
                 The name of the table to retrieve column data about.
-                
+
             [`default`=None] {any}
                 The value to return if no columns are found.
 
@@ -709,7 +711,7 @@ class MySQLDatabase:
         '''
         # if table_name in self._tables:
         #     return self._tables[table_name]['columns']
-        
+
         if self.connect() is False:
             return False
         # @Mstep [] get the INFORMATION_SCHEMA.COLUMNS data from the database.
@@ -722,18 +724,18 @@ class MySQLDatabase:
 
         if isinstance(result,(list)) is False:
             return default
-        
+
         keys = self.get_foreign_keys(table_name)
-        
-        
+
+
         columns = []
         for col in result:
             for key in keys:
                 if col['COLUMN_NAME'] == key['COLUMN_NAME']:
                     col['constraint'] = key
             columns.append(col)
-        
-        
+
+
         # import colemen_utilities.database_utils.MySQL.Column.Column as _col
         output = columns
         # output = []
@@ -754,7 +756,7 @@ class MySQLDatabase:
     #         -------------------------
     #         `table_name` {str}
     #             The name of the table to retrieve column data about.
-                
+
     #         [`default`=None] {any}
     #             The value to return if no columns are found.
 
@@ -774,7 +776,7 @@ class MySQLDatabase:
     #     '''
     #     # if table_name in self._tables:
     #     #     return self._tables[table_name]['columns']
-        
+
     #     if self.connect() is False:
     #         return False
     #     # @Mstep [] get the INFORMATION_SCHEMA.COLUMNS data from the database.
@@ -796,7 +798,11 @@ class MySQLDatabase:
     #     return output
 
     def get_cerberus_schema(self,table_name,crud_type=None,to_json=False):
-        result = self.get_column_data(table_name)
+        tb = self.get_table(table_name)
+        result = []
+        if tb is not None:
+            result = tb.columns
+        # result = self.get_column_data(table_name)
 
         # col:colemen_config._db_column_type = result[0]
 
@@ -867,7 +873,7 @@ class MySQLDatabase:
 
     def get_table_data(self,table_name:str):
         '''
-            Retrieve the Table's meta data and 
+            Retrieve the Table's meta data and
             ----------
 
             Arguments
@@ -916,7 +922,8 @@ class MySQLDatabase:
         if self.connect() is False:
             # @Mstep [RETURN] return False
             return False
-        
+
+        output = None
         # @Mstep [] instantiate the table
         tb = Table.Table(self,table_name)
         # @Mstep [IF] if the table cannot be loaded from a cache file.
@@ -937,9 +944,13 @@ class MySQLDatabase:
                     # tb = Table.Table(self,table_name,table_data)
                     # @Mstep [] add the table to self._tables[table_name]
                     self._tables[table_name] = tb
+                    tb.save_cache()
+                    output = tb
         # @Mstep [ELSE] if the table was loaded from the cache file.
         else:
             self._tables[table_name] = tb
+            output = tb
+        return output
 
     def get_table_meta_data(self,table_name:str,default=None):
         sql = f"""select *
@@ -953,7 +964,7 @@ class MySQLDatabase:
             if isinstance(table,(dict)):
                 table = _obj.keys_to_snake_case(table)
 
-            
+
                 output.append(table)
         if len(output) == 0:
             output = default
@@ -984,20 +995,64 @@ class MySQLDatabase:
     # def validate_to_table(self,table_name:str,data:dict):
     #     cerberus_validate = _obj.get_kwarg(['cerberus_validate'],False,(bool),**kwargs)
     #     crud_type = _obj.get_kwarg(['crud','crud_type'],None,(str),**kwargs)
-    #     default_on_fail = _obj.get_kwarg(['default_on_failure'],False,(bool),**kwargs)        
+    #     default_on_fail = _obj.get_kwarg(['default_on_failure'],False,(bool),**kwargs)
+
+    def __assign_default_meta_cols(self,tb:_db_table_type,crud_type):
+        output = {}
+        if crud_type in ['create']:
+            if tb.has_timestamp_column:
+                output['timestamp'] = datetime.datetime.now(tz=datetime.timezone.utc).timestamp()
+
+            if tb.has_modified_timestamp_column:
+                output['modified_timestamp'] = datetime.datetime.now(tz=datetime.timezone.utc).timestamp()
+
+            if tb.has_hash_id:
+                output['hash_id'] = tb.gen_hash_id()
+
+        if crud_type in ['update']:
+            if tb.has_modified_timestamp_column:
+                output['modified_timestamp'] = datetime.datetime.now(tz=datetime.timezone.utc).timestamp()
+
+        if crud_type in ['delete']:
+            if tb.has_modified_timestamp_column:
+                output['modified_timestamp'] = datetime.datetime.now(tz=datetime.timezone.utc).timestamp()
+
+            if tb.has_deleted_column:
+                output['deleted'] = datetime.datetime.now(tz=datetime.timezone.utc).timestamp()
+
+
+        return output
+
+
+
 
     def correlate_to_table(self,table_name:str,data:dict,**kwargs):
         cerberus_validate = _obj.get_kwarg(['cerberus_validate'],False,(bool),**kwargs)
         crud_type = _obj.get_kwarg(['crud','crud_type'],None,(str),**kwargs)
         default_on_fail = _obj.get_kwarg(['default_on_failure'],False,(bool),**kwargs)
-        cols = self.get_column_data(table_name)
+
+        tb = self.get_table(table_name)
+        cols = tb.columns
+        # cols = self.get_column_data(table_name)
         col:_db_column_type
-        output = {}
+        output = self.__assign_default_meta_cols(tb,crud_type)
+
+
         for col in cols:
             name = col.data.column_name
             if name in data:
                 value = data[name]
                 value_type = type(value).__name__
+
+                # if col.data.is_timestamp:
+                #     value = datetime.datetime.now(tz=datetime.timezone.utc).timestamp()
+                #     output[name] = value
+                #     continue
+                # if col.data.is_hash_id:
+                #     value = datetime.datetime.now(tz=datetime.timezone.utc).timestamp()
+                #     output[name] = value
+                #     continue
+
                 if cerberus_validate:
                     validation_schema = col.validation.cerberus_schema(crud_type)
                     if validation_schema is None:
@@ -1017,7 +1072,7 @@ class MySQLDatabase:
                         _log(f"Successfully validated {name}","success")
                         output[name] = output_data[name]
                         # continue
-                        
+
 
                 if value_type in col.data.py_data_type:
                     output[name] = value
