@@ -238,15 +238,17 @@ def reverse_sanitize_quotes(string):
 
     return new_list
 
-def sanitize_quotes(string):
+def sanitize_quotes(value):
     orig_list = False
-    if isinstance(string, (list)):
+    if isinstance(value,(int,float,bool)) is True:
+        return value
+    if isinstance(value, (list)):
         orig_list = True
-    if isinstance(string, (str)):
-        string = [string]
+    if isinstance(value, (str)):
+        value = [value]
 
     new_list = []
-    for item in string:
+    for item in value:
         if isinstance(item, (str)):
             item = item.replace("'", "&apos_")
             item = item.replace('"', "&quot_")
